#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.assembleDocs = exports.parseDocFiles = void 0;
/* eslint-disable no-console */
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const globby_1 = __importDefault(require("globby"));
const harvester_1 = require("../../../type-harvester/src/harvester");
const deepMergeOneLevel_1 = require("./utils/deepMergeOneLevel");
const contentFromPath_1 = require("./utils/contentFromPath");
const addGeneratedContentToDocs_1 = require("./docModifications/addGeneratedContentToDocs");
const GetTypeMap = async (typePaths, overridePath = '', declarationPaths) => {
    const overrideFileContent = overridePath === '' ? {} : (0, contentFromPath_1.contentFromPath)(overridePath);
    return Promise.all(typePaths.map((typePath) => {
        return (0, globby_1.default)([`${typePath}/**/*.{ts,tsx}`])
            .then((files) => {
            return (0, harvester_1.getProps)(files, overrideFileContent, declarationPaths);
        })
            .catch((error) => {
            throw error;
        });
    })).then((results) => {
        return results.reduce((acc, current) => {
            return (0, deepMergeOneLevel_1.deepMergeOneLevel)(acc, current);
        });
    });
};
const loadDocFiles = (files) => files.sort().map((file) => {
    const filePath = path_1.default.relative(__dirname, file);
    const relativePath = path_1.default.relative(process.cwd(), file);
    return import(filePath)
        .then((module) => {
        const moduleContent = module.default?.default ?? module.default;
        return moduleContent
            ? { ...moduleContent, filePath: relativePath }
            : moduleContent;
    })
        .catch((error) => console.log(error));
});
const parseDocFiles = async (files, basePathForTypes, isLandingPage = false, overridePath = '', typesInputDirs, declarationPaths) => {
    const resultPromises = loadDocFiles(files);
    const allTypePaths = typesInputDirs
        ? [...typesInputDirs, basePathForTypes]
        : [basePathForTypes];
    const typeMapPromise = isLandingPage
        ? null
        : GetTypeMap(allTypePaths, overridePath, declarationPaths);
    const [typeMap, ...docs] = await Promise.all([
        typeMapPromise,
        ...resultPromises,
    ]);
    const nonEmptyDocs = docs.filter((result) => result !== undefined);
    (0, addGeneratedContentToDocs_1.addGeneratedContentToDocs)(nonEmptyDocs, typeMap);
    nonEmptyDocs.forEach((doc) => {
        delete doc.filePath;
    });
    return nonEmptyDocs;
};
exports.parseDocFiles = parseDocFiles;
const assembleDocs = async (inputDir = '', outputDir = '', isLandingPage = false, isCategoryPage = false, overridePath = '', typesInput = '', declarationPath = []) => {
    const baseOutputPath = path_1.default.join(process.cwd(), outputDir);
    const inputDirs = Array.isArray(inputDir) ? inputDir : [inputDir];
    const typesInputDirs = Array.isArray(typesInput) ? typesInput : [typesInput];
    const declarationPathDirs = Array.isArray(declarationPath)
        ? declarationPath
        : [declarationPath];
    return Promise.all(inputDirs.map((dir) => createDocs(dir, typesInputDirs, baseOutputPath, isLandingPage, isCategoryPage, overridePath, declarationPathDirs))).then((results) => {
        const filePath = (results[0] && results[0]?.filePath) ?? '';
        const allResults = results
            .map((result) => result && result?.results)
            .flat();
        fs.mkdirSync(path_1.default.dirname(filePath), { recursive: true });
        fs.writeFileSync(filePath, JSON.stringify(allResults, undefined, 2));
    });
};
exports.assembleDocs = assembleDocs;
const createDocs = (inputDir, typesInputDirs, baseOutputPath, isLandingPage, isCategoryPage, overridePath, declarationPaths) => {
    const baseFilePath = path_1.default.join(process.cwd(), inputDir);
    const baseTypeInputFilePaths = typesInputDirs
        .filter((dir) => dir !== '')
        .map((dir) => path_1.default.join(process.cwd(), dir));
    const fileName = (isLandingPage, isCategoryPage) => {
        if (isCategoryPage)
            return './generated_category_pages.json';
        if (isLandingPage)
            return './generated_static_pages.json';
        return './generated_docs_data.json';
    };
    return (0, globby_1.default)([`${baseFilePath}/**/*.doc.js`, '!**/node_modules/**'])
        .then((files) => {
        return (0, exports.parseDocFiles)(files, baseFilePath, isLandingPage, overridePath, baseTypeInputFilePaths, declarationPaths);
    })
        .then((results) => {
        const filePath = path_1.default.join(baseOutputPath, fileName(isLandingPage, isCategoryPage));
        return { results, filePath };
    })
        .catch((error) => console.error(error));
};
