"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addJSDocExamples = void 0;
const getRelevantTypes_1 = require("../../../../type-harvester/src/getRelevantTypes");
const addCodeblockToExamples_1 = require("../contentModifications/addCodeblockToExamples");
const addJSDocExamples = (doc, typeAST) => {
    const definitions = [];
    if (doc.jsDocTypeExamples) {
        doc.jsDocTypeExamples.forEach((type) => {
            definitions.push({
                type,
                typeDefinitions: (0, getRelevantTypes_1.getRelevantTypes)(typeAST, type, ''),
            });
        });
    }
    let jsDocExampleGroups = [];
    let jsDocExamples = [];
    definitions.forEach((definition) => {
        const type = definition.type;
        const typeDefinition = definition.typeDefinitions[type];
        if (typeDefinition.members) {
            // Type is an interface. Build example groups from its members.
            jsDocExampleGroups = jsDocExampleGroups.concat(extractJsDocExamplesFromMembers(doc, typeDefinition));
        }
        else if (typeDefinition.examples) {
            // Type is something else. Build examples (or default if a single one) from its own examples.
            jsDocExamples = jsDocExamples.concat(convertJSDocExamples(doc, typeDefinition.examples));
        }
        if (typeDefinition.returns &&
            definition.typeDefinitions[typeDefinition.returns.name]) {
            // Function returns an interface. Build example groups from its members.
            jsDocExampleGroups = jsDocExampleGroups.concat(extractJsDocExamplesFromMembers(doc, definition.typeDefinitions[typeDefinition.returns.name]));
        }
    });
    if (jsDocExamples.length > 0) {
        if (jsDocExamples.length === 1 && !doc.defaultExample) {
            doc.defaultExample = jsDocExamples[0];
        }
        else {
            doc.examples = {
                description: '',
                ...(doc.examples || {}),
                examples: [...(doc.examples?.examples || []), ...jsDocExamples],
            };
        }
    }
    if (jsDocExampleGroups.length > 0) {
        doc.examples = {
            description: '',
            ...(doc.examples || {}),
            exampleGroups: [
                ...(doc.examples?.exampleGroups || []),
                ...jsDocExampleGroups,
            ],
        };
    }
};
exports.addJSDocExamples = addJSDocExamples;
function extractJsDocExamplesFromMembers(doc, typeDefinition) {
    return Object.entries(typeDefinition.members)
        .filter(([_index, member]) => member.examples)
        .map(([_index, member]) => {
        const examples = convertJSDocExamples(doc, member.examples);
        return {
            title: member.name,
            examples,
        };
    });
}
const convertJSDocExamples = (doc, typeExamples) => {
    const mappedExamples = typeExamples.map((example) => {
        return {
            description: example.description,
            codeblock: {
                title: example.title,
                tabs: example.tabs.map((tab) => ({
                    title: tab.title,
                    code: tab.code,
                    language: 'typescript',
                })),
            },
        };
    });
    return (0, addCodeblockToExamples_1.addCodeblockToExamples)(mappedExamples, doc.filePath, true);
};
