#!/usr/bin/env node
"use strict";
/* eslint-disable no-console */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTemplate = exports.generateScaffold = exports.scaffoldDocs = void 0;
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const scaffoldDocs = (type = 'entity', output = '', name, id) => {
    if (type !== 'entity' && type !== 'static')
        throw new Error("Error: Please enter a valid type. Valid types are 'entity' or 'static'.");
    if (!name || name === '')
        throw new Error('Please enter a name for your file. Example: --name futureName');
    if (type === 'entity' && id) {
        throw new Error('Reference entity pages do not have an ID. Please remove the ID from the command.');
    }
    if (type === 'static' && !id) {
        throw new Error('Static pages require an ID. Please add an ID to the command. Example: --id static-page-id');
    }
    let staticId;
    if (type === 'static' && id) {
        staticId = id;
    }
    if (type === 'static' && typeof staticId !== 'string') {
        throw new Error('Error: The ID provided is invalid. Ensure it contains only lowercase letters, numbers, and hyphens (no spaces or special characters), and is formatted in kebab-case.');
    }
    const filePath = (0, exports.generateScaffold)(type, output, name, staticId);
    console.log(`File ${name}.doc.ts has been created using the ${type} template. Click the file path to open the new file: ${filePath}`);
};
exports.scaffoldDocs = scaffoldDocs;
const generateScaffold = (type, output, name, staticId) => {
    const baseFilePath = path_1.default.join(process.cwd(), output);
    const filePath = path_1.default.join(baseFilePath, `${name}.doc.ts`);
    fs.mkdirSync(path_1.default.dirname(filePath), { recursive: true });
    const dataContent = (0, exports.createTemplate)(type, staticId);
    fs.writeFileSync(filePath, dataContent);
    return filePath;
};
exports.generateScaffold = generateScaffold;
const createTemplate = (templateSchema, staticId) => {
    const basePath = path_1.default.join(__dirname, './scaffolding');
    const templateContent = fs.readFileSync(path_1.default.join(basePath, `/${templateSchema}.ts`), 'utf8');
    const dataContentWithImport = templateContent.replace(`'../template-schemas/types'`, `'@shopify/generate-docs'`);
    if (staticId) {
        return dataContentWithImport.replace('default-static-page-id', staticId);
    }
    return dataContentWithImport;
};
exports.createTemplate = createTemplate;
