import {ReferenceEntityTemplateSchema} from '@shopify/generate-docs';

// Order of data shape mimics visual structure of page
// Anything in an array can have multiple objects

const data: ReferenceEntityTemplateSchema = {
  // The title of the page.
  name: 'Name of the entity',
  // A description of the reference entity. Can include Markdown.
  description: 'A description of the entity.',
  // Optional. If used will cause the page to only be visible when the feature flag is enabled.
  // featureFlag: 'feature_flag_name',
  // Optional. A short sentence of what is needed to use this entity, such as a version dependency.
  requires: 'version 1.0.0',
  // Optional. What category the entity is: component, hook, utility, etc.
  type: 'component',
  // Boolean that determines if the entity is a visual component.
  isVisualComponent: true,
  // Optional. The example that appears in the right hand column at the top of the page. Represents the primary use case.
  // defaultExample: {
  //   // Optional. If used will cause the card to only be visible when the feature flag is enabled.
  //   featureFlag: 'feature_flag_name',
  //   // Optional. An image preview of the example.
  //   image: 'path/to/image.png',
  //   // The data for the code block.
  //   codeblock: {
  //     // Tabs that appear at the top of the code block.
  //     tabs: [
  //       {
  //         // Optional. The title of the tab.
  //         title: 'Tab 1',
  //         // The relative file path to the code file. Content will be automatically extracted from that file.
  //         code: 'path/to/example.ts',
  //         // Optional. The name of the language of the code.
  //         language: 'typescript',
  //       },
  //     ],
  //     // The title of the code block.
  //     title: 'Code block title',
  //     // Optional. Links to display in the top right corner of the code block.
  //     links: [
  //       {
  //         // Text to display in the tooltip.
  //         name: 'Link name',
  //         // Link url.
  //         url: 'www.github.com',
  //         // Icon name to display.
  //         icon: 'Github',
  //       },
  //     ],
  //   },
  // },
  // Optional. Displays generated TypeScript information, such as prop tables.
  // definitions: [
  //   {
  //     // Title of the list of definitions.
  //     title: 'Definition List Title',
  //     // Description of the definitions. Can use Markdown.
  //     description: 'Description of the definitions.',
  //     // Name of the TypeScript type this entity uses.
  //     type: 'TypeScriptType',
  //   },
  // ],
  // This determines where in the sidebar the entity will appear.
  category: 'Category Name',
  // Optional. Further determines where in the sidebar category an entity will appear.
  subCategory: 'Subcategory Name',
  // // Optional. A thumbnail image to display in the category page.
  // thumbnail: './path/to/thumbnail.png',
  // Optional. A section for examples. Examples may be grouped or ungrouped.
  // examples: {
  //   // Description of the example section. Can use Markdown.
  //   description: 'Description of the examples.',
  //   // Optional. May be used to group examples of a certain theme.
  //   exampleGroups: [
  //     {
  //       // Optional. Title of the example group.
  //       title: 'Example Group Title',
  //       // Optional. If used will cause the example group to only be visible when the feature flag is enabled.
  //       featureFlag: 'feature_flag_name',
  //       examples: [
  //         {
  //           // Optional. Description of the example. Can use Markdown.
  //           description: 'Description of the example.',
  //           // Optional. If used will cause the example group to only be visible when the feature flag is enabled.
  //           featureFlag: 'feature_flag_name',
  //           // Optional. An image preview of the example.
  //           image: './path/to/image.png',
  //           // The data for the code block.
  //           codeblock: {
  //             // Tabs that appear at the top of the code block.
  //             tabs: [
  //               {
  //                 // Optional. The title of the tab.
  //                 title: 'Tab 1',
  //                 // The relative file path to the code file. Content will be automatically extracted from that file.
  //                 code: 'path/to/example.ts',
  //                 // Optional. The name of the language of the code.
  //                 language: 'ruby',
  //               },
  //             ],
  //             // The title of the code block.
  //             title: 'Code block title',
  //             // Optional. Links to display in the top right corner of the code block.
  //             links: [
  //               {
  //                 // Text to display in the tooltip.
  //                 name: 'Additional Link',
  //                 // Link url.
  //                 url: 'www.github.com',
  //                 // Icon name to display.
  //                 icon: 'Github',
  //               },
  //             ],
  //           },
  //         },
  //       ],
  //     },
  //   ],
  //   // Optional. May be used to group examples without a particular theme.
  //   examples: [
  //     {
  //       // Optional. Description of the example. Can use Markdown.
  //       description: '',
  //       // Optional. If used will cause the example group to only be visible when the feature flag is enabled.
  //       featureFlag: '',
  //       // Optional. An image preview of the example.
  //       image: '',
  //       // The data for the code block.
  //       codeblock: {
  //         // Tabs that appear at the top of the code block.
  //         tabs: [
  //           {
  //             // Optional. The title of the tab.
  //             title: '',
  //             // The relative file path to the code file. Content will be automatically extracted from that file.
  //             code: 'path/to/example.ts',
  //             // Optional. The name of the language of the code.
  //             language: '',
  //           },
  //         ],
  //         // The title of the code block.
  //         title: '',
  //         // Optional. Links to display in the top right corner of the code block.
  //         links: [
  //           {
  //             // Text to display in the tooltip.
  //             name: '',
  //             // Link url.
  //             url: '',
  //             // Icon name to display.
  //             icon: '',
  //           },
  //         ],
  //       },
  //     },
  //   ],
  // },
  // A section that displays related entities in a grid of cards.
  related: [
    {
      // Name of the related object/entity/link.
      name: 'Related Entity Name',
      // Optional. A subordinate title providing additional information. The subtitle should be limited to a one word description when in the related section.
      subtitle: 'Subtitle for the related entity',
      // Link to the entity you wish to display in card form.
      url: 'https://www.shopify.dev',
      // Optional. What category the entity is: component, hook, utility, etc. Determines the icon displayed on the card.
      type: 'hook',
      // Optional. If used will cause the card to only be visible when the feature flag is enabled.
      // featureFlag: 'some_random_feature_flag',
    },
  ],
};

export default data;
