import {LandingTemplateSchema} from '@shopify/generate-docs';

// Order of data shape mimics visual structure of page
// Anything in an array can have multiple objects

const data: LandingTemplateSchema = {
  // The title of the page.
  title: 'Landing Page Title',
  // A short description of the page. Appears in the hero section below the title.
  description: 'Description of the landing page.',
  // Optional. If used will cause the page to only be visible when the feature flag is enabled.
  // featureFlag: 'feature_flag_name',
  // Optional. The image used in the hero section. To use a default image, set the value to `/assets/landing-pages/templated-apis/hero.png`.
  // image: '/assets/landing-pages/templated-apis/hero.png',
  // Optional. The image used in the hero section in dark mode.To use a default image, set the value to `/assets/landing-pages/templated-apis/hero-dark.png`.
  // darkImage: '/assets/landing-pages/templated-apis/hero-dark.png',
  // Optional. The tablet image used in the hero section.
  // tabletImage: '/assets/landing-pages/templated-apis/hero.png',
  // Optional. The image used in the hero section in dark mode.
  // tabletDarkImage: '/assets/landing-pages/templated-apis/hero-dark.png',
  // Optional. The image used in the hero section.
  // mobileImage: '/assets/landing-pages/templated-apis/hero.png',
  // Optional. The image used in the hero section in dark mode.
  // mobileDarkImage: '/assets/landing-pages/templated-apis/hero-dark.png',
  // The id for the page that is used for routing. If this documentation is for a primary landing page, confirm id matches the reference name.
  id: 'default-static-page-id',
  // Basic content for the page and Hero section.
  sections: [
    {
      // The generic section is the most flexible content section for landing pages and can be used for everything.
      // Type id for the generic section.
      type: 'Generic',
      // Anchor link for the section.
      anchorLink: 'generic-section',
      // The title of the section.
      title: 'Generic Section Title',
      // Content for the section.
      sectionContent: 'Generic section content.',
      // Optional. An image to display in the section, located on the left side under the code block if provided.
      // image: './path/to/image.png',
      // Optional. Alt text to be provided with the image.
      // altText: 'alt text for image',
      // Optional. A way to author notices at the end of the section content.
      sectionNotice: [
        {
          // The title of the notice.
          title: 'Notice Title',
          // Content for the notice.
          sectionContent: 'Notice content.',
          // The type of the notice. This determines the icon and theming. Options include: 'critical',  'warning', 'info', or 'note'.
          type: 'info',
        },
      ],
      // Optional. A way to author cards at the end of the section content.
      sectionCard: [
        {
          // Name of the related object/entity/link.
          name: 'Section Card Name',
          // Optional. A subordinate title providing additional information.
          subtitle: 'Section Card Subtitle',
          // Link to the entity you wish to display in card form.
          url: 'https://www.github.com',
          // Optional. What category the entity is (e.g., component, hook, utility, etc.). Determines the icon on the card.
          type: 'utility',
          // Optional. If used will cause the card to only be visible when the feature flag is enabled.
          // featureFlag: 'feature_flag_name',
        },
      ],
      // Optional. A way to author subsections inside the section that are separated from the main content by a horizontal line.
      sectionSubContent: [
        {
          // The title of the subsection.
          title: 'Section Sub Content Title',
          // Content that appears in the subsection.
          sectionContent: 'Section sub content.',
          // Optional. A way to author notices at the end of the subsection content.
          sectionNotice: [
            {
              // The title of the notice.
              title: 'Section Notice Title',
              // Content in the notice.
              sectionContent: 'Section notice content.',
              // The type of the notice. This determines the icon and theming. Options include: 'critical',  'warning', 'info', or 'note'.
              type: 'note',
            },
          ],
          // Optional. A way to author cards at the end of the subsection content.
          sectionCard: [
            {
              // Name of the related object/entity/link.
              name: 'Section Card Name',
              // Optional. A subordinate title providing additional information.
              subtitle: 'Section Card Subtitle',
              // Link to the entity you wish to display in card form.
              url: 'https://www.shopify.dev',
              // Optional. What category the entity is: component, hook, utility, etc. Determines the icon on the card.
              type: 'hook',
              // Optional. If used will cause the card to only be visible when the feature flag is enabled.
              // featureFlag: 'feature_flag_name',
            },
          ],
        },
      ],
      // Optional. Code block with multiple tabs that has samples for each language/framework.
      // codeblock: {
      //   // The data for each tab in the code block.
      //   tabs: [
      //     {
      //       // Optional. The title of the tab.
      //       title: 'Tab 1',
      //       // The relative link to the code file.
      //       code: './path/to/code/file.ts',
      //       // Optional. The language of the code file.
      //       language: 'ts',
      //     },
      //   ],
      //   // The title of the code block.
      //   title: 'Code Block Title',
      //   // Optional. Links to display in the top right corner of the code block.
      //   links: [
      //     {
      //       // Text to display in the tooltip.
      //       name: 'Link Name',
      //       // Link url.
      //       url: 'https://www.github.com',
      //       // Icon to display.
      //       icon: 'Github',
      //     },
      //   ],
      // },
      // Optional. The default selected language. If none provided, will default to the first language in the `codeLanguages` array.
      // initialLanguage: 'ts',
    },
    {
      // The generic accordion section is a more flexible section for landing pages and can be used for storing content in an accordion.
      // Type id for the generic accordion section.
      type: 'GenericAccordion',
      // Anchor link for the section.
      anchorLink: 'generic-accordion-section',
      // The title of the section.
      title: 'Generic Accordion Section Title',
      // Optional. Content for the section.
      sectionContent: 'Generic accordion section content.',
      /** The content to be displayed in the accordion. */
      accordionContent: [
        {
          // The title of the accordion item.
          title: 'Accordion Item Title',
          // Description of the accordion item.
          description: 'Accordion item description.',
          // Optional. An image to display in the section, located on the left side under the code block if provided.
          // image: './path/to/image.png',
          // Optional. Alt text to be provided with the image.
          // altText: 'Image alt text',
        },
      ],
    },
    {
      // The resource section contains cards with links to other resources listed in a grid.
      // Type id for the resource section.
      type: 'Resource',
      // Anchor link for the section.
      anchorLink: 'resource-section',
      // The title of the section.
      title: 'Resource Section Title',
      // Content for each card in the section.
      resources: [
        {
          // Name of the related object/entity/link.
          name: 'Resource Name',
          // Optional. A subordinate title providing additional information.
          subtitle: 'Resource Subtitle',
          // Link to the entity you wish to display in card form.
          url: 'https://www.shopify.dev',
          // Optional. What category the entity is (e.g., component, hook, utility, etc.). Determines the icon on the card.
          type: 'component',
          // Optional. If used will cause the card to only be visible when the feature flag is enabled.
          // featureFlag: 'feature_flag_name',
        },
      ],
    },
  ],
};

export default data;
