"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRelevantTypes = void 0;
const ignoredTypes = [
    'React',
    'ReactNode',
    'any',
    'CSSProperties',
    'ElementType',
    'MouseEvent',
    'File',
    'HTMLElement',
];
const nearTypeMatchText = (ast, name) => {
    const splitName = name.split(/(?=[A-Z])/);
    const typeNames = Object.keys(ast);
    const partMatches = splitName
        .map((namePart) => typeNames.filter((astName) => astName.includes(namePart)))
        .flat();
    const partMatchDuplicates = partMatches.filter((item, index) => partMatches.indexOf(item) !== index);
    const matchesForMessage = partMatchDuplicates.length
        ? partMatchDuplicates
        : partMatches;
    return matchesForMessage.length
        ? ` Did you mean: ${matchesForMessage.join(', or ')}?`
        : '';
};
function getRelevantTypes(ast, name, filePath) {
    let matchingNode = ast[name]?.[filePath];
    if (!matchingNode) {
        if (filePath !== '') {
            console.warn(`filePath not found for type ${name} using default value`);
        }
        matchingNode = Object.values(ast[name] ?? {})[0];
    }
    if (!matchingNode) {
        throw new Error(`Type ${name} was not found.${nearTypeMatchText(ast, name)} Expected to find a Props definition at ast['${name}']['${filePath}'] or at least Object.values(ast['${name}])[0]`);
    }
    const pascalCaseRegex = /[A-Z]+[a-z0-9]+(?:[A-Z]+[a-z0-9]+)*/gm;
    const output = {};
    extractTypes(matchingNode);
    function extractTypes(node) {
        const { importMap: notDisplayed, ...displayType } = node;
        output[node.name] = displayType;
        const importMap = node.importMap;
        const listOfMembers = node.members
            ? node.members.map((member) => member.value.toString()).join(' ')
            : node.value.toString();
        const typeDefinitionString = node.params
            ? node.params
                .map((param) => param.value.toString())
                .concat(node.returns?.value.toString() ?? '')
                .join(' ')
            : listOfMembers;
        const detectedTypeDefinitions = typeDefinitionString.match(pascalCaseRegex);
        detectedTypeDefinitions?.forEach((name) => {
            if (ignoredTypes.includes(name))
                return;
            if (Object.keys(output).includes(name))
                return;
            const typeDefinitionInSameFile = ast[name]
                ? ast[name][node.filePath]
                : undefined;
            if (typeDefinitionInSameFile) {
                extractTypes(typeDefinitionInSameFile);
            }
            else if (importMap?.[name] && ast?.[name]?.[importMap[name]]) {
                extractTypes(ast[name][importMap[name]]);
            }
            else {
                const typeDefinitionsWithSameName = ast[name];
                const typeDefinitionsWithSameNameCount = Object.keys(typeDefinitionsWithSameName || {}).length;
                if (typeDefinitionsWithSameNameCount === 1) {
                    extractTypes(Object.values(typeDefinitionsWithSameName)[0]);
                }
                else {
                    console.warn(`Found ${typeDefinitionsWithSameNameCount} definitions for type ${name}`);
                }
            }
        });
    }
    return output;
}
exports.getRelevantTypes = getRelevantTypes;
