"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTypesFromFilePaths = void 0;
const ts = __importStar(require("typescript"));
const normalizePath_1 = require("./utils/normalizePath");
const createImportMap_1 = require("./createImportMap");
const parseEnumDeclaration_1 = require("./parseEnumDeclaration");
const parseInterface_1 = require("./parseInterface");
const parseTypeAlias_1 = require("./parseTypeAlias");
const parseFunctionLikeDeclaration_1 = require("./parseFunctionLikeDeclaration");
const parseClassDeclaration_1 = require("./parseClassDeclaration");
const getTypesFromFilePaths = (program, checker, declarationPaths) => {
    const typeMap = {};
    for (const sourceFile of program.getSourceFiles()) {
        const normalizedPath = (0, normalizePath_1.normalizePath)(sourceFile.fileName, process.cwd());
        if (!sourceFile.isDeclarationFile ||
            declarationPaths?.some((path) => normalizedPath.startsWith(path) ||
                sourceFile.fileName.startsWith(path))) {
            const importMap = (0, createImportMap_1.createImportMap)(sourceFile, checker);
            ts.forEachChild(sourceFile, (child) => {
                visit(typeMap, checker, program, normalizedPath, child, importMap);
            });
        }
    }
    return typeMap;
};
exports.getTypesFromFilePaths = getTypesFromFilePaths;
const visit = (typeMap, checker, program, filePath, node, importMap, level = 0) => {
    // Escape hatch
    if (level > 10) {
        return;
    }
    const nonEmptyImportMap = Object.keys(importMap).length > 0 ? importMap : undefined;
    const parserArgs = [
        typeMap,
        node,
        checker,
        program,
        filePath,
        nonEmptyImportMap,
    ];
    switch (node.kind) {
        case ts.SyntaxKind.EnumDeclaration:
            (0, parseEnumDeclaration_1.parseEnumDeclaration)(...parserArgs);
            break;
        case ts.SyntaxKind.InterfaceDeclaration:
            (0, parseInterface_1.parseInterfaceDeclaration)(...parserArgs);
            break;
        case ts.SyntaxKind.ClassDeclaration:
            (0, parseClassDeclaration_1.parseClassDeclaration)(...parserArgs);
            break;
        case ts.SyntaxKind.TypeAliasDeclaration:
            (0, parseTypeAlias_1.parseTypeAliasDeclaration)(...parserArgs);
            break;
        case ts.SyntaxKind.FunctionDeclaration:
            (0, parseFunctionLikeDeclaration_1.parseFunctionLikeDeclaration)(typeMap, node, checker, program, filePath, nonEmptyImportMap);
            break;
        case ts.SyntaxKind.VariableStatement:
            node.forEachChild((child) => {
                if (ts.isVariableDeclarationList(child)) {
                    child.forEachChild((child) => {
                        if (ts.isVariableDeclaration(child)) {
                            (0, parseFunctionLikeDeclaration_1.parseFunctionLikeDeclaration)(typeMap, child, checker, program, filePath, nonEmptyImportMap);
                        }
                    });
                }
            });
            break;
    }
};
