"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseClassDeclaration = void 0;
const parseJsDocTags_1 = require("./utils/parseJsDocTags");
const parseProperties_1 = require("./utils/parseProperties");
const getSymbolComment_1 = require("./utils/getSymbolComment");
const getClassDetails = (_declaration, type, checker, filePath) => {
    const members = (0, parseProperties_1.parseProperties)(type.getProperties(), filePath, checker);
    let examples;
    if (type.aliasSymbol) {
        examples = (0, parseJsDocTags_1.parseJSDocTags)(type.aliasSymbol).examples;
    }
    else if (type.symbol) {
        examples = (0, parseJsDocTags_1.parseJSDocTags)(type.symbol).examples;
    }
    return { members, examples };
};
const parseClassDeclaration = (ast, node, checker, _program, filePath, importMap) => {
    const declaration = node;
    const name = declaration.name?.getText();
    if (!name) {
        return;
    }
    const type = checker.getTypeAtLocation(declaration);
    const classDetails = getClassDetails(declaration, type, checker, filePath);
    const symbol = checker.getSymbolAtLocation(declaration.name);
    const description = (symbol && (0, getSymbolComment_1.getSymbolComment)(symbol, checker)) ?? '';
    if (!ast[name]) {
        ast[name] = {};
    }
    ast[name][filePath] = {
        filePath,
        ...(importMap && { importMap }),
        name,
        description,
        members: classDetails?.members,
        value: declaration.getText(),
        ...(classDetails?.examples ? { examples: classDetails.examples } : {}),
    };
};
exports.parseClassDeclaration = parseClassDeclaration;
