"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseFunctionLikeDeclaration = void 0;
const ts = __importStar(require("typescript"));
const getSymbolComment_1 = require("./utils/getSymbolComment");
const parseJsDocTags_1 = require("./utils/parseJsDocTags");
const capitalize = (name) => `${name.slice(0, 1).toUpperCase()}${name.slice(1)}`;
const parseFunctionLikeDeclaration = (ast, node, checker, _program, filePath, importMap) => {
    const declaration = node;
    const type = checker.getTypeAtLocation(declaration);
    const functionDetails = getFunctionDetails(declaration, type, checker, filePath);
    let name;
    let description;
    if (declaration.kind === ts.SyntaxKind.TypeAliasDeclaration &&
        declaration.type?.kind === ts.SyntaxKind.FunctionType) {
        name = declaration.name?.getText() ?? '';
        description =
            (type.aliasSymbol && (0, getSymbolComment_1.getSymbolComment)(type.aliasSymbol, checker)) ?? '';
    }
    else {
        name = declaration.name?.getText()
            ? `${capitalize(declaration.name?.getText())}GeneratedType`
            : '';
        description = (type.symbol && (0, getSymbolComment_1.getSymbolComment)(type.symbol, checker)) ?? '';
    }
    if (!ast[name]) {
        ast[name] = {};
    }
    ast[name][filePath] = {
        filePath,
        ...(importMap && { importMap }),
        name,
        description,
        params: functionDetails?.params,
        returns: functionDetails?.returns,
        value: ts.isTypeAliasDeclaration(declaration) &&
            functionDetails.functionFromDeclaration
            ? functionDetails.functionFromDeclaration.getText()
            : declaration.getText(),
        ...(functionDetails?.examples ? { examples: functionDetails.examples } : {}),
    };
};
exports.parseFunctionLikeDeclaration = parseFunctionLikeDeclaration;
const getFunctionFromDeclaration = (declaration) => {
    if (ts.isFunctionDeclaration(declaration)) {
        return declaration;
    }
    const functionLikeChild = declaration
        .getChildren()
        .find((child) => ts.isFunctionLike(child));
    if (functionLikeChild && ts.isFunctionLike(functionLikeChild)) {
        return functionLikeChild;
    }
    return null;
};
const getFunctionDetails = (declaration, type, checker, filePath) => {
    const functionFromDeclaration = getFunctionFromDeclaration(declaration);
    if (!functionFromDeclaration) {
        return {};
    }
    const typeAliases = functionFromDeclaration.typeParameters?.map((typeParameter) => {
        if (!typeParameter.constraint) {
            return null;
        }
        return {
            [typeParameter.name.getText()]: checker.typeToString(checker.getTypeAtLocation(typeParameter.constraint)),
        };
    });
    let examples;
    if (type.aliasSymbol) {
        examples = (0, parseJsDocTags_1.parseJSDocTags)(type.aliasSymbol).examples;
    }
    else if (type.symbol) {
        examples = (0, parseJsDocTags_1.parseJSDocTags)(type.symbol).examples;
    }
    return {
        functionFromDeclaration,
        returns: {
            filePath,
            description: ts
                .getAllJSDocTagsOfKind(functionFromDeclaration, ts.SyntaxKind.JSDocReturnTag)
                .map((tag) => tag.comment)
                .join(''),
            name: parseFunctionReturnType(functionFromDeclaration, checker) ??
                functionFromDeclaration.type?.getText() ??
                '',
            value: parseFunctionReturnType(functionFromDeclaration, checker) ??
                functionFromDeclaration.type?.getText() ??
                '',
        },
        params: functionFromDeclaration.parameters.map((sym, index) => {
            let value = checker.typeToString(checker.getTypeAtLocation(sym));
            const constraintValue = typeAliases?.find((constraintValue) => constraintValue?.[value]);
            if (constraintValue) {
                value = `${value} extends ${constraintValue[value]}`;
            }
            const defaultValue = sym.initializer?.getText();
            const isOptional = Boolean(sym.questionToken || defaultValue);
            return {
                name: parseInputName(sym.name.getText(), index),
                description: ts
                    .getJSDocTags(sym)
                    .map((tag) => tag.comment)
                    .join('')
                    .replace(/^-\s/, ''),
                value,
                ...(isOptional && { isOptional: isOptional ? true : undefined }),
                ...(Boolean(defaultValue) && { defaultValue }),
                filePath,
            };
        }),
        examples,
    };
};
const parseFunctionReturnType = (functionFromDeclaration, checker) => {
    if (!functionFromDeclaration.type ||
        !functionFromDeclaration.typeParameters) {
        return null;
    }
    const isArray = ts.isArrayTypeNode(functionFromDeclaration.type);
    const typeAliasArguments = checker.getTypeAtLocation(isArray
        ? functionFromDeclaration.type.elementType
        : functionFromDeclaration.type).aliasTypeArguments;
    if (ts.isArrayTypeNode(functionFromDeclaration.type)) {
        return parseArrayTypeNodes(functionFromDeclaration.type, checker);
    }
    if (typeAliasArguments) {
        const value = typeAliasArguments
            .map((argument) => parseTypes(argument, checker))
            ?.join(', ');
        const typeText = functionFromDeclaration.type
            .getText()
            .replace(/\r*\t*\s*/g, '');
        const matches = /<\r*\t*\s*(.*)\r*\t*\s*>/g.exec(typeText);
        if (matches && value) {
            const newType = typeText.replace(matches[1], value);
            return isArray ? `${newType}[]` : newType;
        }
    }
    else {
        if (ts.isUnionTypeNode(functionFromDeclaration.type) ||
            ts.isIntersectionTypeNode(functionFromDeclaration.type)) {
            const isUnionType = ts.isUnionTypeNode(functionFromDeclaration.type);
            return functionFromDeclaration.type.types
                .map((typeNode) => {
                const isTypeNodeArray = ts.isArrayTypeNode(typeNode);
                return (parseArrayTypeNodes(typeNode, checker) ??
                    parseTypes(checker.getTypeAtLocation(typeNode), checker, isTypeNodeArray) ??
                    typeNode.getText());
            })
                .join(isUnionType ? ' | ' : ' & ');
        }
        return parseTypes(checker.getTypeAtLocation(functionFromDeclaration.type), checker, isArray);
    }
    return null;
};
const parseTypes = (argument, checker, isArray) => {
    if (isArray) {
        return null;
    }
    if (argument.isClassOrInterface()) {
        return null;
    }
    if (argument.isUnionOrIntersection()) {
        return argument.types
            .map((type) => {
            if (!type.symbol) {
                return checker.typeToString(type, undefined, undefined);
            }
            return type.symbol.declarations?.map((declaration) => {
                if (ts.isInterfaceDeclaration(declaration)) {
                    return declaration.name;
                }
                return declaration.getText();
            });
        })
            .join(` ${argument.isUnion() ? '|' : '&'} `);
    }
    if (argument.symbol) {
        return argument.symbol.declarations
            ?.map((declaration) => {
            return declaration.getText();
        })
            .join(', ');
    }
    return null;
};
const parseInputName = (name, index) => {
    return name.startsWith('{') ? `input${index + 1}` : name;
};
const parseArrayTypeNodes = (typeNode, checker) => {
    if (ts.isArrayTypeNode(typeNode)) {
        const symbol = checker.getTypeAtLocation(typeNode.elementType).symbol;
        if (symbol && symbol.declarations) {
            const firstDeclaration = symbol.declarations[0];
            if (ts.isInterfaceDeclaration(firstDeclaration)) {
                return null;
            }
            return `(${firstDeclaration.getText()})[]`;
        }
    }
    return null;
};
