"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseInterfaceDeclaration = void 0;
const ts = __importStar(require("typescript"));
const createImportMap_1 = require("./createImportMap");
const parseProperties_1 = require("./utils/parseProperties");
const getSymbolComment_1 = require("./utils/getSymbolComment");
const deduplicateMembers_1 = require("./utils/deduplicateMembers");
const collectBaseDescriptions_1 = require("./utils/collectBaseDescriptions");
const parseInterfaceDeclaration = (ast, node, checker, _program, filePath, importMap) => {
    const interfaceDeclaration = node;
    const type = checker.getTypeAtLocation(interfaceDeclaration.name);
    const symbol = checker.getSymbolAtLocation(interfaceDeclaration.name);
    if (!symbol)
        throw new Error('Expected interface declaration to have symbol');
    const members = [];
    // Collect base property descriptions if extending
    let baseDescriptions = {};
    if (interfaceDeclaration.heritageClauses) {
        interfaceDeclaration.heritageClauses.forEach((clause) => {
            if (clause.token === ts.SyntaxKind.ExtendsKeyword) {
                clause.types.forEach((typeNode) => {
                    const baseType = checker.getTypeAtLocation(typeNode);
                    baseDescriptions = {
                        ...baseDescriptions,
                        ...(0, collectBaseDescriptions_1.collectBaseDescriptions)(baseType, checker),
                    };
                });
            }
        });
    }
    // Add inherited members first (from extends, Pick, Omit, etc.)
    const parsedExpression = parseExtendsWithOmits(interfaceDeclaration, filePath, checker);
    members.push(...parsedExpression.types);
    // Then add members declared directly in this interface (these should override inherited ones)
    interfaceDeclaration.members.forEach((member) => {
        if (member.kind === ts.SyntaxKind.IndexSignature) {
            const indexSignature = member;
            const name = `[${indexSignature.parameters
                .map((param) => `${param.name.getText()}: ${param.type?.getText()}`)
                .join(', ')}]`;
            const value = indexSignature.type.getText();
            members.push({ filePath, name, value });
        }
    });
    members.push(...(0, parseProperties_1.parseProperties)(type.getProperties(), filePath, checker, baseDescriptions));
    // sort properties alphabetically
    members.sort((first, second) => first.name.localeCompare(second.name));
    // Deduplicate members so redefinitions override base properties
    const dedupedMembers = (0, deduplicateMembers_1.deduplicateMembers)(members);
    const name = interfaceDeclaration.name.escapedText.toString();
    const description = (0, getSymbolComment_1.getSymbolComment)(symbol, checker);
    const value = interfaceDeclaration.getText();
    if (!ast[name]) {
        ast[name] = {};
    }
    ast[name][filePath] = {
        filePath,
        ...((importMap ||
            Object.keys(parsedExpression.nestedImportMap).length > 0) && {
            importMap: { ...importMap, ...parsedExpression.nestedImportMap },
        }),
        name,
        description,
        members: dedupedMembers,
        value,
    };
};
exports.parseInterfaceDeclaration = parseInterfaceDeclaration;
const parseExtendsWithOmits = (interfaceDeclaration, filePath, checker) => {
    if (!interfaceDeclaration.heritageClauses) {
        return { types: [], nestedImportMap: {} };
    }
    return interfaceDeclaration.heritageClauses.reduce((acc, clause) => {
        if (clause.token === ts.SyntaxKind.ExtendsKeyword) {
            const parsedExpression = clause.types.reduce((acc, type) => {
                const { types, nestedImportMap } = parseTypeExpressionWithArguments(type, checker, filePath);
                // Separate ts.Symbols and custom Type objects
                const symbolTypes = [];
                const customTypes = [];
                types.forEach((t) => {
                    if (t.getName) {
                        symbolTypes.push(t);
                    }
                    else {
                        customTypes.push(t);
                    }
                });
                return {
                    types: [
                        ...acc.types,
                        ...(0, parseProperties_1.parseProperties)(symbolTypes, filePath, checker),
                        ...customTypes,
                    ],
                    nestedImportMap: {
                        ...acc.nestedImportMap,
                        ...nestedImportMap,
                    },
                };
            }, { types: [], nestedImportMap: {} });
            return {
                types: [...acc.types, ...parsedExpression.types],
                nestedImportMap: {
                    ...acc.nestedImportMap,
                    ...parsedExpression.nestedImportMap,
                },
            };
        }
        else {
            return acc;
        }
    }, { types: [], nestedImportMap: {} });
};
const parseTypeExpressionWithArguments = (type, checker, filePath) => {
    const exprText = type.expression.getText();
    if (exprText !== 'Omit' &&
        exprText !== 'Pick' &&
        exprText !== 'Extract' &&
        exprText !== 'Exclude') {
        return { types: [], nestedImportMap: {} };
    }
    if (!type.typeArguments?.[0] || !type.typeArguments?.[1]) {
        return { types: [], nestedImportMap: {} };
    }
    if (exprText === 'Omit' || exprText === 'Pick') {
        const secondArgumentNames = parseSecondArgumentNode(type.typeArguments[1], checker).map((name) => name.replace(/'|"/g, ''));
        const { availableTypes, nestedImportMap } = parsePrimaryNode(type.typeArguments[0], checker);
        if (exprText === 'Omit') {
            return {
                types: availableTypes.filter((prop) => !secondArgumentNames.includes(prop.getName())),
                nestedImportMap,
            };
        }
        if (exprText === 'Pick') {
            return {
                types: availableTypes.filter((prop) => secondArgumentNames.includes(prop.getName())),
                nestedImportMap,
            };
        }
    }
    if (exprText === 'Extract' || exprText === 'Exclude') {
        const value = (0, parseProperties_1.getExtractExcludeTypeValues)(exprText, Array.from(type.typeArguments), checker);
        const mainType = checker.getTypeAtLocation(type.typeArguments[0]);
        const nestedImportMap = mainType.symbol?.declarations?.[0]?.parent &&
            ts.isSourceFile(mainType.symbol?.declarations?.[0]?.parent)
            ? (0, createImportMap_1.createImportMap)(mainType.symbol.declarations[0].parent, checker, true)
            : {};
        return {
            types: [
                {
                    name: exprText,
                    value,
                    filePath,
                },
            ],
            nestedImportMap,
        };
    }
    return { types: [], nestedImportMap: {} };
};
const parsePrimaryNode = (node, checker) => {
    const mainType = checker.getTypeAtLocation(node);
    if (!mainType.symbol?.members) {
        return { availableTypes: [], nestedImportMap: {} };
    }
    return {
        availableTypes: mainType.getProperties(),
        nestedImportMap: createNestedImportMap(mainType, checker),
    };
};
const parseSecondArgumentNode = (node, checker) => {
    if (ts.isLiteralTypeNode(node)) {
        return [node.getText()];
    }
    if (ts.isUnionTypeNode(node)) {
        return node.types.map((type) => type.getText());
    }
    else {
        if (ts.isTypeReferenceNode(node)) {
            const nodeType = checker.getTypeAtLocation(node);
            if (nodeType.isStringLiteral()) {
                return [`'${nodeType.value}'`];
            }
            return (nodeType.aliasSymbol?.getDeclarations()?.flatMap((declaration) => {
                if (ts.isTypeAliasDeclaration(declaration) &&
                    ts.isUnionTypeNode(declaration.type)) {
                    return declaration.type.types.map((type) => type.getText());
                }
                return [];
            }) ?? []);
        }
        return [];
    }
};
const createNestedImportMap = (mainType, checker) => {
    if (mainType.symbol?.declarations?.[0].parent &&
        ts.isSourceFile(mainType.symbol?.declarations?.[0].parent)) {
        return (0, createImportMap_1.createImportMap)(mainType.symbol?.declarations?.[0].parent, checker, true);
    }
    return {};
};
