"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseTypeAliasDeclaration = void 0;
/* eslint-disable no-console */
const ts = __importStar(require("typescript"));
const parseFunctionLikeDeclaration_1 = require("./parseFunctionLikeDeclaration");
const parseProperties_1 = require("./utils/parseProperties");
const getSymbolComment_1 = require("./utils/getSymbolComment");
const createTupleMember_1 = require("./utils/createTupleMember");
const createImportMap_1 = require("./createImportMap");
const deduplicateMembers_1 = require("./utils/deduplicateMembers");
const collectBaseDescriptions_1 = require("./utils/collectBaseDescriptions");
const parseTypeAliasDeclaration = (ast, node, checker, program, filePath, importMap) => {
    const typeAliasDeclaration = node;
    const symbol = checker.getSymbolAtLocation(typeAliasDeclaration.name);
    if (!symbol) {
        throw new Error('Expected type alias declaration to have a symbol');
    }
    const description = (0, getSymbolComment_1.getSymbolComment)(symbol, checker);
    const name = symbol.escapedName.toString();
    const syntaxKind = ts.SyntaxKind[typeAliasDeclaration.kind];
    if (!typeAliasDeclaration.type) {
        console.log(typeAliasDeclaration.getSourceFile().text);
    }
    if (typeAliasDeclaration.type.kind === ts.SyntaxKind.FunctionType) {
        (0, parseFunctionLikeDeclaration_1.parseFunctionLikeDeclaration)(ast, node, checker, program, filePath, importMap);
        return;
    }
    let value = typeAliasDeclaration.type.getText();
    let members;
    if (typeAliasDeclaration.type.kind === ts.SyntaxKind.TypeReference) {
        const type = checker.getTypeAtLocation(typeAliasDeclaration.name);
        if (type.symbol?.members) {
            // Collect base property descriptions
            const baseDescriptions = (0, collectBaseDescriptions_1.collectBaseDescriptions)(type, checker);
            members = (0, parseProperties_1.parseProperties)(type.getProperties(), filePath, checker, baseDescriptions);
        }
        else if (name !== checker.typeToString(type)) {
            value = checker.typeToString(type);
        }
    }
    if (typeAliasDeclaration.type.kind === ts.SyntaxKind.TypeLiteral) {
        const type = checker.getTypeAtLocation(typeAliasDeclaration.name);
        members = (0, parseProperties_1.parseProperties)(type.getProperties(), filePath, checker);
    }
    if (ts.isTupleTypeNode(typeAliasDeclaration.type)) {
        members = typeAliasDeclaration.type.elements.map((element, index) => {
            return (0, createTupleMember_1.createTupleMember)(`[${index}]`, element.getText(), filePath);
        });
    }
    if (typeAliasDeclaration.type.kind === ts.SyntaxKind.UnionType) {
        const unionType = typeAliasDeclaration.type;
        value = unionType.types.map((type) => type.getText()).join(' | ');
    }
    else if (typeAliasDeclaration.type.kind === ts.SyntaxKind.IntersectionType) {
        const unionType = typeAliasDeclaration.type;
        value = unionType.types.map((type) => type.getText()).join(' & ');
    }
    const parsedExpression = parseTypeReferenceForKeyWords(typeAliasDeclaration, checker);
    // Custom type guard for member types
    function isMemberTypes(arr) {
        return (Array.isArray(arr) &&
            arr.length > 0 &&
            typeof arr[0].getName === 'function');
    }
    // Type guard for Extract/Exclude result
    function isExtractExcludeArray(arr) {
        return (Array.isArray(arr) && arr.length > 0 && typeof arr[0].value === 'string');
    }
    if (members && isMemberTypes(parsedExpression.types)) {
        members.push(...(0, parseProperties_1.parseProperties)(parsedExpression.types, filePath, checker));
    }
    else if (isMemberTypes(parsedExpression.types)) {
        members = (0, parseProperties_1.parseProperties)(parsedExpression.types, filePath, checker);
    }
    else if (isExtractExcludeArray(parsedExpression.types)) {
        value = parsedExpression.types[0].value;
    }
    if (!ast[name]) {
        ast[name] = {};
    }
    // sort properties alphabetically
    members?.sort((first, second) => first.name.localeCompare(second.name));
    if (members) {
        members = (0, deduplicateMembers_1.deduplicateMembers)(members);
    }
    ast[name][filePath] = {
        filePath,
        ...((importMap ||
            Object.keys(parsedExpression.nestedImportMap).length > 0) && {
            importMap: { ...importMap, ...parsedExpression.nestedImportMap },
        }),
        syntaxKind,
        name,
        value,
        description,
        ...(members && { members }),
    };
};
exports.parseTypeAliasDeclaration = parseTypeAliasDeclaration;
const hasOmit = (typeAliasDeclaration) => {
    return typeAliasDeclaration
        .getChildren()
        .filter((child) => ts.isTypeReferenceNode(child))
        .some((item) => ts.isTypeReferenceNode(item) &&
        item.typeName.getFullText().includes('Omit'));
};
const parseTypeReferenceForKeyWords = (typeAlias, checker) => {
    const type = typeAlias
        .getChildren()
        .filter((child) => ts.isTypeReferenceNode(child))?.[0];
    if (!type) {
        return { types: [], nestedImportMap: {} };
    }
    const typeText = type.getText();
    if (!(typeText.includes('Omit') ||
        typeText.includes('Pick') ||
        typeText.includes('Extract') ||
        typeText.includes('Exclude'))) {
        return { types: [], nestedImportMap: {} };
    }
    if (!type.typeArguments?.[0] || !type.typeArguments?.[1]) {
        return { types: [], nestedImportMap: {} };
    }
    // Omit/Pick logic
    const secondArgumentNames = parseSecondArgumentNode(type.typeArguments[1], checker).map((name) => name.replace(/'|"/g, ''));
    const { availableTypes, nestedImportMap } = parsePrimaryNode(type.typeArguments[0], checker);
    if (typeText.includes('Omit')) {
        return {
            types: availableTypes.filter((prop) => !secondArgumentNames.includes(prop.getName())),
            nestedImportMap,
        };
    }
    if (typeText.includes('Pick')) {
        return {
            types: availableTypes.filter((prop) => secondArgumentNames.includes(prop.getName())),
            nestedImportMap,
        };
    }
    // Extract/Exclude logic
    if (typeText.includes('Extract') || typeText.includes('Exclude')) {
        const exprText = type.typeName.getText();
        const value = (0, parseProperties_1.getExtractExcludeTypeValues)(exprText, Array.from(type.typeArguments), checker);
        return {
            types: [
                {
                    name: exprText,
                    value,
                    filePath: '',
                },
            ],
            nestedImportMap: {},
        };
    }
    return { types: [], nestedImportMap: {} };
};
const parsePrimaryNode = (node, checker) => {
    const mainType = checker.getTypeAtLocation(node);
    if (!mainType.symbol?.members) {
        return { availableTypes: [], nestedImportMap: {} };
    }
    return {
        availableTypes: mainType.getProperties(),
        nestedImportMap: createNestedImportMap(mainType, checker),
    };
};
const parseSecondArgumentNode = (node, checker) => {
    if (ts.isLiteralTypeNode(node)) {
        return [node.getText()];
    }
    if (ts.isUnionTypeNode(node)) {
        return node.types.map((type) => type.getText());
    }
    else {
        if (ts.isTypeReferenceNode(node)) {
            const nodeType = checker.getTypeAtLocation(node);
            if (nodeType.isStringLiteral()) {
                return [`'${nodeType.value}'`];
            }
            return (nodeType.aliasSymbol?.getDeclarations()?.flatMap((declaration) => {
                if (ts.isTypeAliasDeclaration(declaration) &&
                    ts.isUnionTypeNode(declaration.type)) {
                    return declaration.type.types.map((type) => type.getText());
                }
                return [];
            }) ?? []);
        }
        return [];
    }
};
const createNestedImportMap = (mainType, checker) => {
    if (mainType.symbol?.declarations?.[0].parent &&
        ts.isSourceFile(mainType.symbol?.declarations?.[0].parent)) {
        return (0, createImportMap_1.createImportMap)(mainType.symbol?.declarations?.[0].parent, checker, true);
    }
    return {};
};
