"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseProperties = exports.getExtractExcludeTypeValues = void 0;
const ts = __importStar(require("typescript"));
const getSymbolComment_1 = require("./getSymbolComment");
const parseJsDocTags_1 = require("./parseJsDocTags");
const getExtractExcludeTypeValues = (typeName, typeArguments, checker) => {
    if (typeArguments.length !== 2)
        return '';
    const [tArg, uArg] = typeArguments;
    const tType = checker.getTypeAtLocation(tArg);
    const uType = checker.getTypeAtLocation(uArg);
    let unionTypes = [];
    if (tType.isUnion()) {
        unionTypes = tType.types;
    }
    else {
        unionTypes = [tType];
    }
    // Fallback: compare string representations for string literal types
    const uStrs = checker.typeToString(uType).split(' | ');
    const filteredTypes = unionTypes.filter((typeItem) => {
        const tStr = checker.typeToString(typeItem);
        return typeName === 'Extract'
            ? uStrs.includes(tStr)
            : !uStrs.includes(tStr);
    });
    return filteredTypes
        .map((t) => checker.typeToString(t).replace(/"/g, "'"))
        .join(' | ');
};
exports.getExtractExcludeTypeValues = getExtractExcludeTypeValues;
const parseProperties = (properties, filePath, checker, baseDescriptions) => {
    return properties.reduce((members, property) => {
        const valueDeclaration = property.valueDeclaration;
        if (!valueDeclaration) {
            return members;
        }
        const ignoredModifiers = [
            ts.SyntaxKind.PrivateKeyword,
            ts.SyntaxKind.ProtectedKeyword,
        ];
        if (ts.canHaveModifiers(valueDeclaration)) {
            const kinds = ts
                .getModifiers(valueDeclaration)
                ?.map((modifier) => modifier.kind);
            if (kinds?.filter((modifier) => ignoredModifiers.includes(modifier)).length) {
                return members;
            }
        }
        const name = property.getName();
        let description = (0, getSymbolComment_1.getSymbolComment)(property, checker);
        if ((!description || description.trim() === '') &&
            baseDescriptions &&
            baseDescriptions[name]) {
            description = baseDescriptions[name];
        }
        const syntaxKind = ts.SyntaxKind[valueDeclaration.kind];
        const type = checker.getTypeOfSymbolAtLocation(property, valueDeclaration);
        let value = getTypeNameFromValueDeclaration(valueDeclaration) ??
            checker.typeToString(type, undefined, ts.TypeFormatFlags.NoTruncation);
        if (ts.isPropertySignature(valueDeclaration) &&
            valueDeclaration.type &&
            ts.isTypeReferenceNode(valueDeclaration.type)) {
            const typeName = valueDeclaration.type.typeName.getText();
            if ((typeName === 'Extract' || typeName === 'Exclude') &&
                valueDeclaration.type.typeArguments?.length === 2) {
                value = (0, exports.getExtractExcludeTypeValues)(typeName, Array.from(valueDeclaration.type.typeArguments), checker);
            }
        }
        const { deprecationMessage, isProtected, defaultValue, isPrivate, environmentValue, examples, } = (0, parseJsDocTags_1.parseJSDocTags)(property);
        const memberNode = {
            filePath,
            syntaxKind,
            name,
            value,
            description,
        };
        if (valueDeclaration.kind === ts.SyntaxKind.PropertySignature ||
            valueDeclaration.kind === ts.SyntaxKind.MethodSignature) {
            const signature = valueDeclaration;
            if (signature.questionToken !== undefined) {
                memberNode.isOptional = true;
            }
        }
        if (deprecationMessage) {
            memberNode.deprecationMessage = deprecationMessage;
        }
        if (isProtected) {
            memberNode.isProtected = true;
        }
        if (isPrivate) {
            memberNode.isPrivate = true;
        }
        if (environmentValue) {
            memberNode.environmentValue = environmentValue;
        }
        if (defaultValue) {
            memberNode.defaultValue = defaultValue;
        }
        if (examples) {
            memberNode.examples = examples;
        }
        return members.concat(memberNode);
    }, []);
};
exports.parseProperties = parseProperties;
const getTypeNameFromValueDeclaration = (valueDeclaration) => {
    const isUnionOrParentheticalUnion = ts.isPropertySignature(valueDeclaration) &&
        valueDeclaration.type &&
        (ts.isUnionTypeNode(valueDeclaration.type) ||
            (ts.isParenthesizedTypeNode(valueDeclaration.type) &&
                ts.isUnionTypeNode(valueDeclaration.type.type)));
    if (ts.isPropertySignature(valueDeclaration) &&
        valueDeclaration.type &&
        (ts.isTypeReferenceNode(valueDeclaration.type) ||
            isUnionOrParentheticalUnion)) {
        return valueDeclaration.type.getText();
    }
    return undefined;
};
